// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package v1

import (
	"github.com/gin-gonic/gin"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jaryhe/gopkgs/tasker/httptasker"
	"github.com/jaryhe/gopkgs/util"
	"go.uber.org/zap"
	"net/http"
	"smart-government-management-gateway/errors"
	param_v1 "smart-government-management-gateway/param/v1"
	"smart-government-management-gateway/pb"
	"smart-government-management-gateway/pb/v1"
	"smart-government-management-gateway/utils"
)



// 告警列表
// @Summary 告警列表
// @Description 告警列表
// @Tags alarm
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param page query int false " "
// @Param start query int64 false " "
// @Param end query int64 false " "
// @Param filter query string false " 设备sn码，设备名，项目名 "
// @Param page_size query int false " "
// @Param device_code query int false " "
// @Param project_id query int false " "
// @Success 200 {object} v1.AlarmListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/alarm/list [get]
func (c *Controller) AlarmList(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.AlarmListRequest{}
	var loginUid int64
	var userName string
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, &req.AlarmListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := param_v1.AlarmListResponse{}
		rpcReq := &v1.AlarmListRequest{
			Page:      req.Page,
			Start:     req.Start,
			End:       req.End,
			IsHandle:2,
			PageSize:req.PageSize,
			Filter:req.Filter,
			ProjectId:req.ProjectId,
			DeviceCode:req.DeviceCode,
		}

		rpcRsp, err := pb.Government.AlarmList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Government.AlarmList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data = *rpcRsp
		if resp.Data.List == nil {
			resp.Data.List = []*v1.AlarmItem{}
		}
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)

}

// 处理告警
// @Summary 处理告警
// @Description 处理告警
// @Tags alarm
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.AlarmHandleBody true " "
// @Success 200 {object} v1.AlarmHandleResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/alarm [put]
func (c *Controller) AlarmHandle(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.AlarmHandleRequest{}
	var loginUid int64
	var userName string
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.AlarmHandleBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := param_v1.AlarmHandleResponse{}
		rpcReq := &v1.AlarmHandleRequest{
			Id:req.Id,
			Handle:req.Handle,
		}

		_, err := pb.Government.AlarmHandle(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Government.AlarmHandle"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)

}


//
// @Summary 告警统计
// @Description 告警统计
// @Tags alarm
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Success 200 {object} v1.AlarmStatisticResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/alarm/statistic [get]
func (c *Controller) AlarmStatistic(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.AlarmStatisticRequest{}
	resp := param_v1.AlarmStatisticResponse{}
	var loginUid int64
	var userName string
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleProjectTask := func() error {
		// 响应数据
		rpcReq := &v1.AlarmProjectRequest{

		}

		rpcRsp, err := pb.Government.AlarmProject(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Government.AlarmProject"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data.ProjectStatisticData = *rpcRsp
		if resp.Data.ProjectStatisticData.List == nil {
			resp.Data.ProjectStatisticData.List = []*v1.AlarmProjectItem{}
		}
		return nil
	}

	handleDeviceTask := func() error {
		// 响应数据
		rpcReq := &v1.AlarmDeviceStatisticRequest{

		}

		rpcRsp, err := pb.Government.AlarmDeviceStatistic(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Government.AlarmDeviceStatistic"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data.DeviceStatisticData = *rpcRsp
		return nil
	}

	handleIncreaseTask := func() error {
		// 响应数据
		rpcReq := &v1.AlarmIncreaseStatisticRequest{

		}

		rpcRsp, err := pb.Government.AlarmIncreaseStatistic(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Government.AlarmIncreaseStatistic"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data.IncreaseStatisticData = *rpcRsp
		return nil
	}

	handleTrendTask := func() error {
		// 响应数据
		rpcReq := &v1.AlarmPercentStatisticRequest{

		}

		rpcRsp, err := pb.Government.AlarmPercentStatistic(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Government.AlarmPercentStatistic"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data.TrendStatisticData = *rpcRsp
		if resp.Data.TrendStatisticData.List == nil {
			resp.Data.TrendStatisticData.List = []*v1.AlarmPercentStaticticItem{}
		}
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleProjectTask, handleDeviceTask, handleIncreaseTask, handleTrendTask)

}

