// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package v1

import (
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jaryhe/gopkgs/tasker/httptasker"
	"github.com/jaryhe/gopkgs/util"
	"net/http"
	"smart-government-management-gateway/errors"
	param_v1 "smart-government-management-gateway/param/v1"
	"smart-government-management-gateway/pb"
	"smart-government-management-gateway/pb/v1"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
	"smart-government-management-gateway/utils"
)

const (
	// 0 供应商入驻审批 1 设备对接审批 2 设备入库审批 3 拆机审批
	ApproveTypeProvider = iota
	ApproveTypeDeviceDocking
	ApproveTypeDeviceDel
)

const (
	// 0 企业入驻审批 2 项目入驻审批 3 项目变更审批
	ApproveTypeCompany = iota
	ApproveTypeProject
	ApproveTypeProjectChange
)

// 供应商注册审批
// @Summary 供应商注册审批
// @Description 供应商注册审批
// @Tags provider
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Param body body v1.ApproveBody true " "
// @Success 200 {object} v1.ApproveResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/provider/approve [put]
func (c *Controller) ProviderApprove(ctx *gin.Context) {
	// 解析参数
	var loginUid int64
	var userName string
	req := &param_v1.ApproveRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.ApproveBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {

		// 响应数据
		resp := param_v1.ApproveResponse{}
		rpcReq := &v1.ProviderApproveRequest{
			ApproveType:ApproveTypeProvider,
			Status:req.Status,
			Feedback:req.Feedback,
			Id:req.Id,
		}

		_, err := pb.Government.ProviderApprove(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Government.ProviderApprove"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return err
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	err := httptasker.Exec(ctx, parseParamTask, handleServiceTask)
	s, _ := json.MarshalToString(req)
	utils.LogWrite("供应商注册审批", loginUid, userName,  s, err, 0)
}

// 设备对接审批
// @Summary 设备对接审批
// @Description 设备对接审批
// @Tags device
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Param body body v1.ApproveBody true " "
// @Success 200 {object} v1.ApproveResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/device/type_approve [put]
func (c *Controller) DeviceTypeApprove(ctx *gin.Context) {
	// 解析参数
	var loginUid int64
	var userName string
	req := &param_v1.ApproveRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.ApproveBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {

		// 响应数据
		isdevelop, err  := utils.GetDevelop(ctx)
		if err != nil {
			return err
		}

		resp := param_v1.ApproveResponse{}
		rpcReq := &v1.ProviderApproveRequest{
			ApproveType:ApproveTypeDeviceDocking,
			Status:req.Status,
			Feedback:req.Feedback,
			Id:req.Id,
			IsDevelop:isdevelop,
		}

		_, err = pb.Government.ProviderApprove(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Government.ProviderApprove"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return err
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	err := httptasker.Exec(ctx, parseParamTask, handleServiceTask)
	s, _ := json.MarshalToString(req)
	utils.LogWrite("设备对接政府审批", loginUid, userName,  s, err, 0)
}

// 拆机审批
// @Summary 拆机审批
// @Description 拆机审批
// @Tags device
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Param body body v1.ApproveBody true " "
// @Success 200 {object} v1.ApproveResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/device/del_approve [put]
func (c *Controller) DeviceDelApprove(ctx *gin.Context) {
	// 解析参数
	var loginUid int64
	var userName string
	req := &param_v1.ApproveRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.ApproveBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	var projectId int64
	handleServiceTask := func() error {

		// 响应数据
		resp := param_v1.ApproveResponse{}
		rpcReq := &v1.ProviderApproveRequest{
			ApproveType:ApproveTypeDeviceDel,
			Status:req.Status,
			Feedback:req.Feedback,
			Id:req.Id,
		}

		reply, err := pb.Government.ProviderApprove(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Government.ProviderApprove"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return err
		}
		projectId = reply.ProjectId
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	err := httptasker.Exec(ctx, parseParamTask, handleServiceTask)
	s, _ := json.MarshalToString(req)
	utils.LogWrite("拆机审批", loginUid, userName,  s, err, projectId)
}

// 开发者审批对接设备
// @Summary 开发者审批对接设备
// @Description 开发者审批对接设备
// @Tags develop
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Param body body v1.ApproveBody true " "
// @Success 200 {object} v1.ApproveResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/develop/docking_approve [put]
func (c *Controller) DockingApprove(ctx *gin.Context) {
	// 解析参数
	var loginUid int64
	var userName string
	req := &param_v1.ApproveRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.ApproveBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {

		// 响应数据
		resp := param_v1.ApproveResponse{}
		rpcReq := &v1.DockingApproveRequest{
			Status:req.Status,
			Comment:req.Feedback,
			Id:req.Id,
		}

		_, err := pb.Government.DockingApprove(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Government.DockingApprove"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return err
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	err := httptasker.Exec(ctx, parseParamTask, handleServiceTask)
	s, _ := json.MarshalToString(req)
	utils.LogWrite("设备对接开发者审批", loginUid, userName,  s, err,0)
}

// 公司注册审批
// @Summary 公司注册审批
// @Description 公司注册审批
// @Tags company
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Param body body v1.ApproveBody true " "
// @Success 200 {object} v1.ApproveResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company/approve [put]
func (c *Controller) CompanyApprove(ctx *gin.Context) {
	// 解析参数
	var loginUid int64
	var userName string
	req := &param_v1.ApproveRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.ApproveBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {

		// 响应数据
		resp := param_v1.ApproveResponse{}
		rpcReq := &v1.CompanyApproveRequest{
			ApproveType:ApproveTypeCompany,
			Status:req.Status,
			Feedback:req.Feedback,
			Id:req.Id,
		}

		_, err := pb.Government.CompanyApprove(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Government.CompanyApprove"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return err
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	err := httptasker.Exec(ctx, parseParamTask, handleServiceTask)
	s, _ := json.MarshalToString(req)
	utils.LogWrite("公司注册审批", loginUid, userName,  s, err, 0)
}
/*
// 项目添加审批
// @Summary 项目添加审批
// @Description 项目添加审批
// @Tags project
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Param body body v1.ApproveBody true " "
// @Success 200 {object} v1.ApproveResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/project/approve [put]
func (c *Controller) ProjectAddApprove(ctx *gin.Context) {
	// 解析参数
	var loginUid int64
	var userName string
	req := &param_v1.ApproveRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.ApproveBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {

		// 响应数据
		resp := param_v1.ApproveResponse{}
		rpcReq := &v1.CompanyApproveRequest{
			ApproveType:ApproveTypeProject,
			Status:req.Status,
			Feedback:req.Feedback,
			Id:req.Id,
		}

		_, err := pb.Government.CompanyApprove(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Government.CompanyApprove"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return err
		}

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	err := httptasker.Exec(ctx, parseParamTask, handleServiceTask)
	s, _ := json.MarshalToString(req)
	utils.LogWrite("项目添加审批", loginUid, userName,  s, err)
}
*/

// 项目变更审批
// @Summary 项目变更审批
// @Description 项目变更审批
// @Tags project
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Param body body v1.ApproveBody true " "
// @Success 200 {object} v1.ApproveResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/project/change_approve [put]
func (c *Controller) ProjectChangeApprove(ctx *gin.Context) {
	// 解析参数
	var loginUid int64
	var userName string
	req := &param_v1.ApproveRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.ApproveBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	var projectId int64
	handleServiceTask := func() error {

		// 响应数据
		resp := param_v1.ApproveResponse{}
		rpcReq := &v1.CompanyApproveRequest{
			ApproveType:ApproveTypeProjectChange,
			Status:req.Status,
			Feedback:req.Feedback,
			Id:req.Id,
		}

		reply, err := pb.Government.CompanyApprove(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Government.CompanyApprove"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return err
		}
		projectId = reply.ProjectId
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	err := httptasker.Exec(ctx, parseParamTask, handleServiceTask)
	s, _ := json.MarshalToString(req)
	utils.LogWrite("项目变更审批", loginUid, userName,  s, err, projectId)
}


