// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package v1

import (

	"github.com/jaryhe/gopkgs/logger"
	"github.com/jaryhe/gopkgs/tasker/httptasker"
	"github.com/jaryhe/gopkgs/util"
	"net/http"
	"fmt"
	"smart-government-management-gateway/consts"
	"smart-government-management-gateway/errors"
	param_v1 "smart-government-management-gateway/param/v1"
	"smart-government-management-gateway/pb"
	"smart-government-management-gateway/pb/v1"
	"strconv"
	"strings"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
	"smart-government-management-gateway/utils"
)

// 公司列表
// @Summary 公司列表
// @Description 公司列表
// @Tags company
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Param page query int true "page"
// @Param filter query string false " "
// @Success 200 {object} v1.CompanyListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company/list [get]
func (c *Controller) CompanyList(ctx *gin.Context) {
	// 解析参数
	var loginUid int64
	var userName string
	req := &param_v1.CompanyListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.CompanyListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {

		// 响应数据
		resp := param_v1.CompanyListResponse{}
		rpcReq := &v1.CompanyListRequest{
			Page:req.Page,
			Filter:req.Filter,
		}
		req.FilterStatus = "1"
		if req.FilterStatus != "" {
			array := strings.Split(req.FilterStatus, ",")
			for _, v := range array {
				value, _ := strconv.Atoi(v)
				rpcReq.FilterStatus = append(rpcReq.FilterStatus, int32(value))
			}
		}
		if rpcReq.Page < 1 {
			rpcReq.Page = 1
		}

		rpcRsp, err := pb.Government.CompanyList(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Government.CompanyList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return err
		}
		resp.Data = *rpcRsp
		if resp.Data.List == nil {
			resp.Data.List = make([]*v1.CompanyItem, 0)
		}
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// 申请的公司列表
// @Summary 申请的公司列表
// @Description 申请的公司列表
// @Tags company
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Param page query int true "page"
// @Param filter query string false " "
// @Success 200 {object} v1.CompanyListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company/apply_list [get]
func (c *Controller) CompanyApplyList(ctx *gin.Context) {
	// 解析参数
	var loginUid int64
	var userName string
	req := &param_v1.CompanyListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.CompanyListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {

		// 响应数据
		resp := param_v1.CompanyListResponse{}
		rpcReq := &v1.CompanyListRequest{
			Page:req.Page,
			Filter:req.Filter,
		}
		req.FilterStatus = fmt.Sprintf("%d,%d", consts.StatusAuditedFail, consts.StatusAuditting)
		if req.FilterStatus != "" {
			array := strings.Split(req.FilterStatus, ",")
			for _, v := range array {
				value, _ := strconv.Atoi(v)
				rpcReq.FilterStatus = append(rpcReq.FilterStatus, int32(value))
			}
		}
		if rpcReq.Page < 1 {
			rpcReq.Page = 1
		}

		rpcRsp, err := pb.Government.CompanyList(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Government.CompanyList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return err
		}
		resp.Data = *rpcRsp
		if resp.Data.List == nil {
			resp.Data.List = make([]*v1.CompanyItem, 0)
		}
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// 项目列表
// @Summary 项目列表
// @Description 项目列表
// @Tags project
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Param page query int false "page"
// @Param page_size query int false "page_size"
// @Param filter query string false " "
// @Success 200 {object} v1.ProjectListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/project/list [get]
func (c *Controller) ProjectList(ctx *gin.Context) {
	// 解析参数
	var loginUid int64
	var userName string
	req := &param_v1.ProjectListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.ProjectListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {

		// 响应数据
		resp := param_v1.ProjectListResponse{}
		rpcReq := &v1.ProjectListRequest{
			Page:req.Page,
			Filter:req.Filter,
			PageSize:req.PageSize,
		}
		if rpcReq.Page < 1 {
			rpcReq.Page = 1
		}

		rpcRsp, err := pb.Government.ProjectList(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Government.ProjectList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return err
		}
		resp.Data = *rpcRsp
		if resp.Data.List == nil {
			resp.Data.List = make([]*v1.ProjectItem, 0)
		}
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// 项目变更申请列表
// @Summary 项目变更申请列表
// @Description 项目变更申请列表
// @Tags project
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Param page query int true "page"
// @Param filter_status query string false "状态，逗号分隔"
// @Param filter query string false " "
// @Success 200 {object} v1.ProjectChangeInfoListResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/project/change_job_list [get]
func (c *Controller) ProjectChangeInfoList(ctx *gin.Context) {
	// 解析参数
	var loginUid int64
	var userName string
	req := &param_v1.ProjectChangeInfoListRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.ProjectChangeInfoListQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {

		// 响应数据
		resp := param_v1.ProjectChangeInfoListResponse{}
		rpcReq := &v1.ProjectChangeInfoListRequest{
			Page:req.Page,
			Filter:req.Filter,
		}
		if req.FilterStatus != "" {
			array := strings.Split(req.FilterStatus, ",")
			for _, v := range array {
				value, _ := strconv.Atoi(v)
				rpcReq.FilterStatus = append(rpcReq.FilterStatus, int32(value))
			}
		}
		if rpcReq.Page < 1 {
			rpcReq.Page = 1
		}

		rpcRsp, err := pb.Government.ProjectChangeInfoList(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Government.ProjectChangeInfoList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return err
		}
		resp.Data = *rpcRsp
		if resp.Data.List == nil {
			resp.Data.List = make([]*v1.ProjectChangeInfoItem, 0)
		}
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 项目统计
// @Description 项目统计
// @Tags 前台
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Success 200 {object} v1.ProjectStatisticResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/project/statistic [get]
func (c *Controller) ProjectStatistic(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ProjectStatisticRequest{}
	var loginUid int64
	var userName string
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据

		resp := param_v1.ProjectStatisticResponse{}
		rpcReq := &v1.ProjectStatisticRequest{}

		reply, err := pb.Government.ProjectStatistic(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "ProjectStatistic"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data = *reply
		if resp.Data.List == nil {
			resp.Data.List = []*v1.ProjectTypeStatisticItem{}
		}
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}


//
// @Summary 项目类型统计
// @Description 项目类型统计
// @Tags 前台
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Success 200 {object} v1.ProjectTypeStatisticResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/project/type_statistic [get]
func (c *Controller) ProjectTypeStatistic(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ProjectTypeStatisticRequest{}
	var loginUid int64
	var userName string
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据

		resp := param_v1.ProjectTypeStatisticResponse{}
		rpcReq := &v1.ProjectTypeStatisticRequest{}

		reply, err := pb.Government.ProjectTypeStatistic(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "ProjectTypeStatistic"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if reply.List == nil {
			reply.List = []*v1.ProjectTypeStatisticItem{}
		}
		resp.Data = *reply
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 员工统计
// @Description 员工统计
// @Tags 前台
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Success 200 {object} v1.StaffStatisticResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/staff/statistic [get]
func (c *Controller) StaffStatistic(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.StaffStatisticRequest{}
	var loginUid int64
	var userName string
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据

		resp := param_v1.StaffStatisticResponse{}
		rpcReq := &v1.StaffStatisticRequest{}

		reply, err := pb.Government.StaffStatistic(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "StaffStatistic"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data = *reply
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}


//
// @Summary 企业统计
// @Description 企业统计
// @Tags 前台
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Success 200 {object} v1.CompanyStatisticResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/company/statistic [get]
func (c *Controller) CompanyStatistic(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyStatisticRequest{}
	var loginUid int64
	var userName string
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		loginUid, userName, err = utils.GetJwtIdFromCtx(ctx)
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据

		resp := param_v1.CompanyStatisticResponse{}
		rpcReq := &v1.CompanyStatisticRequest{}

		reply, err := pb.Government.CompanyStatistic(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "CompanyStatistic"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data = *reply
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}


